"""Load metadata about metadata attributes from JSON files"""

from .load_attribute_data import (
    MDIMPORTER_ATTRIBUTE_DATA,
    MDITEM_ATTRIBUTE_AUDIO,
    MDITEM_ATTRIBUTE_COMMON,
    MDITEM_ATTRIBUTE_DATA,
    MDITEM_ATTRIBUTE_FILESYSTEM,
    MDITEM_ATTRIBUTE_IMAGE,
    MDITEM_ATTRIBUTE_READ_ONLY,
    MDITEM_ATTRIBUTE_SHORT_NAMES,
    MDITEM_ATTRIBUTE_VIDEO,
    NSURL_RESOURCE_KEY_DATA,
)

__all__ = [
    "MDIMPORTER_ATTRIBUTE_DATA",
    "MDITEM_ATTRIBUTE_AUDIO",
    "MDITEM_ATTRIBUTE_COMMON",
    "MDITEM_ATTRIBUTE_DATA",
    "MDITEM_ATTRIBUTE_FILESYSTEM",
    "MDITEM_ATTRIBUTE_IMAGE",
    "MDITEM_ATTRIBUTE_READ_ONLY",
    "MDITEM_ATTRIBUTE_SHORT_NAMES",
    "MDITEM_ATTRIBUTE_VIDEO",
    "NSURL_RESOURCE_KEY_DATA",
]
